<CsoundSynthesizer>
<CsOptions>
-m0
</CsOptions>
<CsInstruments>
sr = 44100
ksmps = 32
nchnls = 2
0dbfs = 1

/* joachim heintz
   SPECTRUM ANALYZER
   
   Analyzes the N loudest bins at a certain position
   of a sound file and prints out the result.
   
   For TIEMF workshop, feb/mar 2021
   
   USAGE:
   1. Insert input file name, number of partials and time just below.
   2. Run the file. A short part of the original sound is being played at the selected start.
   3. Find result in console output, and hear simple additive resynthesis.
   
   NOTE:
   The result needs manual adjustments, as it is a raw FFT snapshot (no partial tracking).
*/


/**********************************************/
/*********** INSERT YOUR INPUT HERE!***********/
/**********************************************/

gSfile = "glass.wav" ;input file (first channel is used for analysis)
giNumParts = 11 ;how many partials to analyse
giTime = .05 ;at which time in the input file

/**********************************************/
/**********************************************/


//UDOs for table sort

  opcode TabMinVal_k, k, ii
ift, iftlen xin
kminval   tab       0, ift
kndx      =         1  
loop:
kval      tab       kndx, ift
kmaxval   =         (kval < kminval ? kval : kminval)
          loop_lt   kndx, 1, iftlen, loop
          xout      kminval
  endop

  opcode TabMaxValIndx_k, kk, iikk
;returns the maximum value of a function table and its index
ift, iftlen, kstrt, kshft xin; ftable, length, startindex, how many values to shift for the next value
kndx      =         kstrt
kresultindx =       kstrt
kresultval tab      kstrt, ift; first value in the table as comparision
loop:
kval      tab       kndx, ift
if kval > kresultval then
kresultindx =	      kndx
kresultval =        kval
endif
          loop_lt   kndx, kshft, iftlen, loop
          xout      kresultval, kresultindx
  endop
  
  opcode TabIndxNMax, 0, iiiii
;returns the indices of the first khowmany largest values in iftsrc and writes them in iftdest
;bin 0 is omitted (shift value in TabMaxValIndx_k)
iftsrc, isrclen, iftcpy, iftdest, ihowmany xin
          tablecopy iftcpy, iftsrc ;leave iftsrc (= amp table) untouched
kmin      TabMinVal_k iftcpy, isrclen; smallest value in iftsrc
kwritindx =         0
loop:
kval, kndx TabMaxValIndx_k iftcpy, isrclen, 1, 1; gets index for maximum
          tabw      kmin-1, kndx, iftcpy ;replace this maximum by kmin-1
          tabw      kndx, kwritindx, iftdest ;write the index to index table
          loop_lt   kwritindx, 1, ihowmany, loop
  endop


//analyse spectrum for strongest partials ...

instr AnalyzeSpectrum
  
 aSig[] diskin gSfile, 1, giTime
 iFFTsize = 1024
 giFtAmps ftgen 0, 0, iFFTsize/2+1, 2, 0
 giFtAmpsCpy ftgen 0, 0, iFFTsize/2+1, 2, 0
 giFtFreqs ftgen 0, 0, iFFTsize/2+1, 2, 0
 giFtRes ftgen 0, 0, giNumParts, 2, 0
 fSig pvsanal aSig[0], iFFTsize, iFFTsize/4, iFFTsize, 1
 kWritten pvsftw fSig, giFtAmps, giFtFreqs
 kCount init 1
 if kWritten == 1 then
  if kCount == 5 then
   TabIndxNMax giFtAmps, ftlen(giFtAmps), giFtAmpsCpy, giFtRes, giNumParts
   schedulek "PrintResult", 0, .1
   turnoff
  endif
  kCount += 1
 endif
 
endin

//... print out the result to console ...

instr PrintResult
 ;put amp and freq result in two arrays
 giAmpArr[] init giNumParts
 giFreqArr[] init giNumParts
 indx = 0
 while indx < giNumParts do
  iBin table indx, giFtRes
  giAmpArr[indx] table iBin, giFtAmps
  giFreqArr[indx] table iBin, giFtFreqs
  indx += 1
 od
 ;look for lowest frequency and create proportions from it
 iMinFreq minarray giFreqArr
 iPropArr[] = giFreqArr / iMinFreq
 ;final printout 
 printf_i "\nFile = '%s'\n", 1, gSfile
 printf_i "Position in file (sec) = %f\n", 1, giTime
 printf_i "Frequency of first partial = %f (MIDI %.2f)\n", 1, iMinFreq, ftom:i(iMinFreq) 
 printf_i "Partial Proportions and Amplitudes for %d strongest Bins:\n", 1, giNumParts
 SProps sprintf "%s","iFreqProps[] fillarray"
 SAmps sprintf "%s", "iAmps[] fillarray"
 indx = 0
 while indx < giNumParts do
  if indx < giNumParts-1 then
   SProps strcat SProps, sprintf(" %f,",iPropArr[indx])
   SAmps strcat SAmps, sprintf(" %f,",giAmpArr[indx])
  else
   SProps strcat SProps, sprintf(" %f",iPropArr[indx])
   SAmps strcat SAmps, sprintf(" %f",giAmpArr[indx])  
  endif
  indx += 1
 od
 printf "%s\n", 1, SProps
 printf "%s\n\n", 1, SAmps
 turnoff
 ;let us listen the result
 schedule "PlayResult", 0, 1/2
endin

// ... and play it

instr PlayResult
 ;at first play original sound
 aSnd[] diskin gSfile, 1, giTime
 aOut linen aSnd[0], .003, p3, p3
 out aOut, aOut
 ;then the analysis 
 indx = 0
 while indx < lenarray(giAmpArr) do
  schedule "PlayPartial", 1, 3, giAmpArr[indx], giFreqArr[indx]
  indx += 1
 od
endin

instr PlayPartial
 iAmp = p4
 iFreq = p5
 aEnv transeg iAmp, p3, -3, 0
 aPartial poscil aEnv, iFreq
 out aPartial, aPartial
endin

</CsInstruments>
<CsScore>
i "AnalyzeSpectrum" 0 4
</CsScore>
</CsoundSynthesizer>
<bsbPanel>
 <label>Widgets</label>
 <objectName/>
 <x>978</x>
 <y>232</y>
 <width>320</width>
 <height>240</height>
 <visible>true</visible>
 <uuid/>
 <bgcolor mode="nobackground">
  <r>255</r>
  <g>255</g>
  <b>255</b>
 </bgcolor>
 <bsbObject version="2" type="BSBLineEdit">
  <objectName>bla</objectName>
  <x>86</x>
  <y>108</y>
  <width>100</width>
  <height>25</height>
  <uuid>{abaac4da-a83e-4398-9dbd-2b08d379c8dc}</uuid>
  <visible>true</visible>
  <midichan>0</midichan>
  <midicc>-3</midicc>
  <description/>
  <label>asdfer</label>
  <alignment>left</alignment>
  <font>Liberation Sans</font>
  <fontsize>10</fontsize>
  <precision>3</precision>
  <color>
   <r>0</r>
   <g>0</g>
   <b>0</b>
  </color>
  <bgcolor mode="nobackground">
   <r>239</r>
   <g>239</g>
   <b>239</b>
  </bgcolor>
  <background>nobackground</background>
 </bsbObject>
</bsbPanel>
<bsbPresets>
</bsbPresets>
